@@ -25,7 +25,6 @@
 package htsjdk.samtools.util;
 
 import htsjdk.HtsjdkTest;
-import htsjdk.samtools.SAMException;
 import htsjdk.samtools.SAMFileHeader;
 import htsjdk.samtools.SAMSequenceRecord;
 import htsjdk.variant.vcf.VCFFileReader;
@@ -36,7 +35,12 @@ import org.testng.annotations.Test;
 import java.io.File;
 import java.io.IOException;
 import java.nio.file.Path;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.LinkedList;
+import java.util.List;
 
 /**
  * Tests the IntervalList class
@@ -44,7 +48,7 @@ import java.util.*;
 public class IntervalListTest extends HtsjdkTest {
 
     final SAMFileHeader fileHeader;
-    final IntervalList list1, list2, list3;
+    final IntervalList list1, list2, list3, empty;
     static final public Path TEST_DIR = new File("src/test/resources/htsjdk/samtools/intervallist").toPath();
 
     public IntervalListTest() {
@@ -54,6 +58,8 @@ public class IntervalListTest extends HtsjdkTest {
         list1 = new IntervalList(fileHeader);
         list2 = new IntervalList(fileHeader);
         list3 = new IntervalList(fileHeader);
+        empty = new IntervalList(fileHeader);
+
 
         list1.add(new Interval("1", 1, 100));     //de-facto: 1:1-200 1:202-300     2:100-150 2:200-300
         list1.add(new Interval("1", 101, 200));
@@ -238,6 +244,21 @@ public class IntervalListTest extends HtsjdkTest {
                 CollectionUtil.makeCollection(list.iterator()));
     }
 
+
+    @Test
+    public void testUnionSamePosition() {
+        final IntervalList iList= new IntervalList(fileHeader);
+
+        final List<Interval> intervals = Arrays.asList(
+                new Interval("1", 2, 100, true, "test1"),
+                new Interval("1", 2, 100, true, "test2")
+        );
+        iList.addall(intervals);
+        final List<Interval> uniqued = iList.uniqued().getIntervals();
+        Assert.assertEquals(uniqued.size(),1);
+        Assert.assertEquals(uniqued.get(0).getName(),"test1|test2");
+    }
+
     @DataProvider(name = "invertData")
     public Object[][] invertData() {
         final IntervalList invert1 = new IntervalList(fileHeader);
@@ -392,17 +413,14 @@ public class IntervalListTest extends HtsjdkTest {
 
         three_overlaps_two.add(new Interval("1", 25, 400));
         three_overlaps_two.add(new Interval("2", 200, 600));
-        //three_overlaps_two.add(new Interval("3", 50, 470));
 
         three_overlaps_one.add(new Interval("1", 25, 400));
         three_overlaps_one.add(new Interval("2", 200, 600));
-        //three_overlaps_one.add(new Interval("3", 50, 470));
 
         one_overlaps_three.add(new Interval("1", 1, 100));
         one_overlaps_three.add(new Interval("1", 101, 200));
         one_overlaps_three.add(new Interval("1", 202, 300));
         one_overlaps_three.add(new Interval("2", 200, 300));
-        //one_overlaps_three.add(new Interval("2", 100, 150));
 
         return new Object[][]{
                 new Object[]{list1, list1, list1}, // should return itself
@@ -410,7 +428,9 @@ public class IntervalListTest extends HtsjdkTest {
                 new Object[]{list2, list1, two_overlaps_one},
                 new Object[]{list3, list2, three_overlaps_two},
                 new Object[]{list3, list1, three_overlaps_one},
-                new Object[]{list1, list3, one_overlaps_three}
+                new Object[]{list1, list3, one_overlaps_three},
+                new Object[]{empty, list1, empty},
+                new Object[]{list1, empty, empty},
         };
     }
 
@@ -420,7 +440,6 @@ public class IntervalListTest extends HtsjdkTest {
 
         three_overlaps_one_and_two.add(new Interval("1", 25, 400));
         three_overlaps_one_and_two.add(new Interval("2", 200, 600));
-        //three_overlaps_one_and_two.add(new Interval("3", 50, 470));
 
         return new Object[][]{
                 new Object[]{CollectionUtil.makeList(list3), CollectionUtil.makeList(list1, list2), three_overlaps_one_and_two},
@@ -448,18 +467,6 @@ public class IntervalListTest extends HtsjdkTest {
                 CollectionUtil.makeCollection(list.iterator()));
     }
 
-    @Test(expectedExceptions = SAMException.class)
-    public void testOverlapsEmptyFirstList() {
-        IntervalList.overlaps(Collections.emptyList(), Collections.singletonList(list1));
-    }
-
-    @Test
-    public void testOverlapsEmptySecondList() {
-        Assert.assertEquals(
-                CollectionUtil.makeCollection(IntervalList.overlaps(Collections.singletonList(list1), Collections.emptyList()).iterator()),
-                Collections.emptyList());
-    }
-
     @DataProvider(name = "VCFCompData")
     public Object[][] VCFCompData() {
         final Path intervalListFromVcf = TEST_DIR.resolve("IntervalListFromVCFTest.vcf");
@@ -476,7 +483,6 @@ public class IntervalListTest extends HtsjdkTest {
     @Test(dataProvider = "VCFCompData")
     public void testFromVCF(final Path vcf, final Path compInterval, final boolean invertVCF) {
 
-
         final IntervalList compList = IntervalList.fromPath(compInterval);
         final IntervalList list = invertVCF ? IntervalList.invert(VCFFileReader.toIntervalList(vcf)) : VCFFileReader.toIntervalList(vcf);
 
@@ -549,22 +555,47 @@ public class IntervalListTest extends HtsjdkTest {
         Assert.assertEquals(test.getIntervals(), CollectionUtil.makeList(new Interval(referenceName, 1, length)));
     }
 
-    @Test
-    public void testMerges() {
-        final SortedSet<Interval> intervals = new TreeSet<Interval>() {{
-            add(new Interval("1", 500, 600, false, "foo"));
-            add(new Interval("1", 550, 650, false, "bar"));
-            add(new Interval("1", 625, 699, false, "splat"));
-        }};
+    @DataProvider
+    public Object[][] getMergeTestCases() {
+        final String contig = "1";
+        final Interval foo = new Interval(contig, 500, 600, false, "foo");
+        final Interval bar = new Interval(contig, 550, 650, false, "bar");
+        final Interval splat = new Interval(contig, 625, 699, false, "splat");
+        final List<Interval> threeInOrderIntervals = Arrays.asList(foo, bar, splat);
+        final List<Interval> threeOutOfOrderIntervals = Arrays.asList(bar, foo, splat);
+        final Interval zeroLengthInterval = new Interval(contig, 626, 625);
+        final Interval interval600To601 = new Interval(contig, 600, 601);
+        final Interval interval600To625 = new Interval(contig, 600, 625);
+        final Interval normalInterval = new Interval(contig, 626, 629, true, "whee");
+        final Interval zeroInterval10To9 = new Interval(contig, 10, 9);
+        return new Object[][]{
+                {threeInOrderIntervals, true, new Interval(contig, 500, 699, false, "foo|bar|splat")},
+                {threeInOrderIntervals, false, new Interval(contig, 500, 699, false, "foo")},
+                {threeOutOfOrderIntervals, true, new Interval(contig, 500, 699, false, "bar|foo|splat")},
+                {threeOutOfOrderIntervals, false, new Interval(contig, 500, 699, false, "bar")},
+                {Collections.singletonList(normalInterval), true, normalInterval},
+                {Collections.singletonList(normalInterval), false, normalInterval},
+                {Collections.singletonList(zeroLengthInterval), true, zeroLengthInterval},
+                {Collections.singletonList(zeroLengthInterval), false, zeroLengthInterval},
+                {Arrays.asList(zeroLengthInterval, interval600To601), true, interval600To625},
+                {Arrays.asList(zeroLengthInterval, interval600To601), false, interval600To625},
+                {Arrays.asList(zeroLengthInterval, interval600To601), true, interval600To625},
+                {Arrays.asList(interval600To601, new Interval(contig, 100, 200, false, "hasName")), true, new Interval(contig, 100, 601, false, "hasName")},
+                {Arrays.asList(interval600To601, new Interval(contig, 100, 200, false, "hasName")), false, new Interval(contig, 100, 601, false, "hasName")},
+                {Arrays.asList(zeroInterval10To9, new Interval(contig, 11, 15)), false, new Interval(contig, 10, 15)},
+                {Arrays.asList(zeroInterval10To9, new Interval(contig, 10, 15)), true, new Interval(contig, 10, 15)},
+                {Arrays.asList(zeroInterval10To9, new Interval(contig, 9,15)), false, new Interval(contig, 9, 15)},
+                {Arrays.asList(zeroInterval10To9, new Interval(contig, 8, 9)), true, new Interval(contig, 8, 9)}
+        };
+    }
 
-        Interval out = IntervalList.merge(intervals, false);
-        Assert.assertEquals(out.getStart(), 500);
-        Assert.assertEquals(out.getEnd(), 699);
-
-        intervals.add(new Interval("1", 626, 629, false, "whee"));
-        out = IntervalList.merge(intervals, false);
-        Assert.assertEquals(out.getStart(), 500);
-        Assert.assertEquals(out.getEnd(), 699);
+    @Test(dataProvider = "getMergeTestCases")
+    public void testMerges(Iterable<Interval> intervals, boolean concatNames, Interval expected) {
+        final Interval merged = IntervalList.merge(intervals, concatNames);
+        Assert.assertEquals(merged.getContig(), expected.getContig());
+        Assert.assertEquals(merged.getStart(), expected.getStart());
+        Assert.assertEquals(merged.getStrand(), expected.getStrand());
+        Assert.assertEquals(merged.getName(), expected.getName());
     }
 
     @Test
@@ -617,15 +648,15 @@ public class IntervalListTest extends HtsjdkTest {
 
     @Test(expectedExceptions = IllegalArgumentException.class)
     public void changeHeader() {
-        SAMFileHeader clonedHeader = fileHeader.clone();
+        final SAMFileHeader clonedHeader = fileHeader.clone();
         clonedHeader.addSequence(new SAMSequenceRecord("4", 1000));
-        IntervalList usingClone1 = new IntervalList(clonedHeader);
+        final IntervalList usingClone1 = new IntervalList(clonedHeader);
         usingClone1.add(new Interval("4", 1, 100));
-        IntervalList usingClone2 = new IntervalList(clonedHeader);
+        final IntervalList usingClone2 = new IntervalList(clonedHeader);
         usingClone2.add(new Interval("4", 10, 20));
 
 
-        IntervalList expected = new IntervalList(clonedHeader);
+        final IntervalList expected = new IntervalList(clonedHeader);
         expected.add(new Interval("4", 1, 9));
         expected.add(new Interval("4", 21, 100));
 
@@ -634,10 +665,7 @@ public class IntervalListTest extends HtsjdkTest {
 
         //now interval lists are in "illegal state" since they contain contigs that are not in the header.
         //this next step should fail
-        IntervalList.subtract(usingClone1, usingClone2);
-
-        Assert.assertTrue(false);
-
+        IntervalList.union(usingClone1, usingClone2);
     }
 
     @Test public void uniqueIntervalsWithoutNames() {
